

#    SAGE (LyrTech) Run Control Application Implementation file

    package require DataAcquisitionControlServer   ;# load generic code

set Sage_DefaultOption ""
set ReSyncTime 0

# do what's needed to setup all hardware, leave in "stopped" state state - 0
proc do_Implementation-SetUp {} {
       global MIDAS_DataAcquisitionCode MIDAS_DataAcquisitionState MIDAS_DataAcquisitionReason
       global MIDAS_DataAcquisitionError MIDAS_DataAcquisitionInfo

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Entered SAGE do_Implementation-SetUp"}

        set z [catch {inform sys do_Setup} m]
        if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "SAGE do_Implementation-SetUp returned z=$z and $m"}

        if {$z != 0} {

            if {[info procs ::DefineMessage] == "::DefineMessage"} {
               ::DefineMessage -c LyrTech -l INFORMATION  "SETUP returned with an error\n$m"
            } 

            puts "SETUP returned with an error\n$m"

            set MIDAS_DataAcquisitionCode 6
        } else {

            if {[info procs ::DefineMessage] == "::DefineMessage"} {
               ::DefineMessage -c LyrTech -l INFORMATION  "SETUP returned $m"
            } 
            set MIDAS_DataAcquisitionCode 0
        }
}

# do what's needed to reset all hardware, leave in "reset" state - 2 (setup needed)
proc do_Implementation-Reset {} {
     global MIDAS_DataAcquisitionCode

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Entered SAGE do_Implementation-Reset"}

     set z [catch {inform sys do_Reset} m]
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "SAGE do_Implementation-Reset returned z=$z and $m"}

     set MIDAS_DataAcquisitionCode 2
}

# do what's needed to stop all "going" Lyrtech streams, leave in "halted" state - 0
proc do_Implementation-Stop {} {
     global MIDAS_DataAcquisitionCode

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Entered SAGE do_Implementation-Stop"}

     set z [catch {inform sys do_Stop} m]
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "SAGE do_Implementation-Stop returned z=$z and $m"}

     set MIDAS_DataAcquisitionCode 0
}

# do what's needed to start all halted Lyrtech streams, leave in "going" state - 1
proc do_Implementation-Go {} {
     global MIDAS_DataAcquisitionCode

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Entered SAGE do_Implementation-Go"}

     set z [catch {inform sys do_Go} m]
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "SAGE do_Implementation-Go returned z=$z and $m"}

     set MIDAS_DataAcquisitionCode 1
}

# do what's needed to set Aata Acq program control options
# Option value comes in through MIDAS_DataAcquisitionOption
# Option name comes in through the argument string, if no name given set the default option
# Special case "ReSyncTime" calls routine to write resynctime to hardware ready for next resync signal
proc do_Implementation-SetOption {args} {
     global MIDAS_DataAcquisitionOption MIDAS_DataAcquisitionCode

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Entered SAGE do_Implementation-SetOption with option \"$args\", value $MIDAS_DataAcquisitionOption"}

     if {[llength $args] > 0 && [llength [first $args]] != 0} then {
        set option [first $args]
     } else {
        set option "Sage_DefaultOption"
     }
     upvar #0 $option thisOption

     set thisOption $MIDAS_DataAcquisitionOption
     if {$option == "ReSyncTime"} then {
       if {$MIDAS_DataAcquisitionCode != 0} {
               set MIDAS_DataAcquisitionError 4097
               set MIDAS_DataAcquisitionInfo "currently in wrong state for requested action"
               return 1
       }
       Write_ReSyncTime
     }
}

# Return current value of option named in argument string, if no name given read the default option
# Option value passed back through MIDAS_DataAcquisitionOption
proc do_Implementation-GetOption {args} {
     global MIDAS_DataAcquisitionOption

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Entered SAGE do_Implementation-GetOption with option \"$args\""}

     if {[llength $args] > 0 && [llength [first $args]] != 0} then {
        set option [first $args]
     } else {
        set option "Sage_DefaultOption"
     }
     upvar #0 $option thisOption

     set z [catch {set value $thisOption}]
     if {$z == 0} then {
       if {[llength $value] == 1} {
         if {[llength [first $value]] == 0} {
           set MIDAS_DataAcquisitionOption ""
         } else {
           set MIDAS_DataAcquisitionOption [first $value]
         }
       } else {
         set MIDAS_DataAcquisitionOption $value
       }
     } else {
         set MIDAS_DataAcquisitionOption "Unknown option \"$option\""
     }
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "Returning \"$MIDAS_DataAcquisitionOption\" from option $option"}
}

# Not implemented
proc do_Implementation-Counters {} {
       global MIDAS_DataAcquisitionCounters
}

# Not implemented
proc do_Implementation-Rates {} {
       global MIDAS_DataAcquisitionRates
}


# Write the value in ReSyncTime to the ReSync registers on all LyrTech boards
proc Write_ReSyncTime {} {
    global ReSyncTime

     global MIDAS_DataAcquisitionControlServer_Verbose;
     if {$MIDAS_DataAcquisitionControlServer_Verbose} {puts "In Write_ReSyncTime with time $ReSyncTime ([format 0x%08x $ReSyncTime])"}

    puts "Now writing $ReSyncTime to LyrTech boards"
}

package provide SageRunControl 1.0

puts "SAGE (LyrTech) Run Control Implementation"
