
#    MIDAS Access Server  for  TDRI module 

package require MIDASDomainHandler       ;#  load default support procedures

package require TDRIAccess ;#  load TDRI access library


global MIDAS_TDRIAccessServer_Verbose
set MIDAS_TDRIAccessServer_Verbose 0

global MIDAS_SOAPDomainHandler_Verbose
set MIDAS_SOAPDomainHandler_Verbose 0

global MIDAS_RegisterAccessServer_Verbose
set MIDAS_RegisterAccessServer_Verbose 0

upvar #0 TDRIAccessServer_Connected Connected
upvar #0 TDRIAccessServer_Xport Xport

set Connected 0

puts "Using TDRI Module with Xport at $Xport"


# Use namespaces to isolate your methods
namespace eval urn:TDRIAccessServer {

    proc AccessServerVerbose {option} {
        global MIDAS_TDRIAccessServer_Verbose
        global MIDAS_RegisterAccessServer_Verbose
        global MIDAS_SOAPDomainHandler_Verbose

        upvar #0 TDRIAccessServer_Connected Connected

        set MIDAS_TDRIAccessServer_Verbose $option
        set MIDAS_RegisterAccessServer_Verbose $option
        set MIDAS_SOAPDomainHandler_Verbose $option

        return [rpcvar::rpcvar int 0]
    }

    proc Init {} {
        set vl [do_Init]
        return [rpcvar::rpcvar int $vl]
    }

    proc WriteControl {Value} {
        do_WriteControl $Value
        return [rpcvar::rpcvar int 0]
    }

    proc ReadControl {} {
        set vl [do_ReadControl]
        return [rpcvar::rpcvar int $vl]
    }

    proc Status {} {
        set vl [do_Status]
        return [rpcvar::rpcvar int $vl]
    }

    proc Version {} {
        set vl [do_Version]
        return [rpcvar::rpcvar int $vl]
    }

    proc do_Init {} {
        upvar #0 TDRIAccessServer_Connected Connected

        set Connected 0

        return 1
    }


    proc do_WriteControl {Value} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::WriteControl"}

#    now start access

        do_Connect

#    do action

       TDRIAccess_WriteControl $Value

        return    ;# completed 
    }

    proc do_ReadControl {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::ReadControl"}

#    now start access

        do_Connect

#    do action

       set vl [TDRIAccess_ReadControl]

        return $vl   ;# completed 
    }

    proc do_Status {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::Status"}

#    now start access

        do_Connect

#    do action

       set vl [TDRIAccess_Status]

        return  $vl   ;# completed 
    }

    proc do_Version {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose

        if {$Verbose} {puts "called method TDRIAccessServer::Version"}

#    now start access

        do_Connect

#    do action

       set vl [TDRIAccess_Version]

        return  $vl   ;# completed 
    }

proc do_Connect {} {
        upvar #0 MIDAS_TDRIAccessServer_Verbose Verbose
        upvar #0 TDRIAccessServer_Xport Xport
        upvar #0 TDRIAccessServer_Connected Connected

#    check status of TCP connection to Xport interface for the TDRI module

        if {$Connected != 0} {return}

            package require TDRIAccess

            TDRIAccess_Verbose $Verbose

            set XportHost [first [split $Xport :]]
            set XportPort [second [split $Xport :]]
            TDRIAccess_Configure -n $XportHost -p $XportPort  

            TDRIAccess_Init

            if {$Verbose} {puts "Access to Xport for TDRI module at $XportHost:$XportPort initialised"}

            set Connected 1
}


    # We have to publish the public methods...

     SOAP::export AccessServerVerbose
     SOAP::export Init
     SOAP::export WriteControl
     SOAP::export ReadControl
     SOAP::export Status
     SOAP::export Version

     
}   ;#  end of    namespace eval urn:TDRIAccessServer

# register this service with tclhttpd
puts [SOAP::Domain::register  -prefix  /TDRIAccessServer -namespace urn:TDRIAccessServer -uri urn:TDRIAccessServer]


proc ::SOAP::Domain::MIDAS_SOAPDomainHandler_/TDRIAccessServer {optsname sock args} {   \
    ::SOAP::Domain::MIDAS_SOAPDomainHandler_/DefaultAccessServer $optsname $sock $args}


package provide TDRIAccessServer 1.0
