#    MIDAS help Control Task

set started 0

set frame HELP
set frametitle "MIDAS Help System and On-Line Manual"
set icontitle Help

set framenumber 1

set rowsize 80

if {[info exists env(Midaspsviewer)]} then {
   set psviewer $env(Midaspsviewer)
} else {
   if {$env(platform) == "unix"} then {set psviewer "ghostview"} else {set psviewer "gsview32"}
}

if {[info exists env(Midasprnviewer)]} then {
   set prnviewer $env(Midasprnviewer)
} else {
   if {$env(platform) == "unix"} then {set prnviewer "ghostview"} else {set prnviewer "gsview32"}
}

if {[info exists env(Midasdviviewer)]} then {
   set dviviewer $env(Midasdviviewer)
} else {
   set dviviewer "xdvi"
}

if {[info exists env(Midasdocumentbase)]} then {
   set documentbase $env(Midasdocumentbase)
} else {
   set documentbase [file join $env(MIDASBASE) documents]
}

proc help-on-widget {task widget} {
# insert-debug "help-on-widget $task $widget"
   set found 0
   foreach format [list html htm txt dvi ps prn gif bmp] {
      set file [findfile $task $widget $format]
      if {$file != ""} then {
         [set format]_show $task $file
         incr found
         continue
      }
   }
   if {$found == 0} then {set found [help-on-task $task]}
   return $found
}

proc help-on-task {task} {
# insert-debug "help-on-task $task"
   set found 0
   foreach format [list html htm txt dvi ps prn gif bmp] {
      set file [findfile $task "" $format]
      if {$file != ""} then {
         [set format]_show $task $file
         incr found
      }
   }
   return $found
}

proc check-for-help-on-task {task} {
   foreach format [list html htm txt dvi ps prn gif bmp] {
      set file [findfile $task "" $format]
      if {$file != ""} then {return 1}
   }
   return 0
}

proc help-on-topic {task topic} {
#  insert-debug "help-on-topic $task $topic"
   set found 0
   foreach format [list html htm txt dvi ps prn gif bmp] {
      set file [findfile $task $topic $format]
      if {$file != ""} then {
         [set format]_show $task $file
         incr found
      }
   }
   if {$found == 0} then {set found [help-on-task $task]}
   return $found
}

proc findfile {taskname topic ftype} {
   global env

   if {$env(platform) == "unix"} then {set J ":"} else {set J ";"}

   set path [split $env(TCLPATH) $J]
   set text ""
   foreach dir $path {
      set root [file join [file dirname $dir] manual]
#    try using a file pathname of the format manual/task.topic.type or manual/task.type
      if {$topic != ""} then {
         set file [file join $root $taskname.$topic.$ftype]
      } else {
         set file [file join $root $taskname.$ftype]
      }
#   insert-debug "help:  looking for file $file"
      if {[file exists $file] && [file isfile $file]} {return $file}

#    try using a file pathname of the format manual/task/topic.type or manual/task/type
      if {$topic != ""} then {
         set file [file join $root $taskname $topic.$ftype]
      } else {
         set file [file join $root $taskname $ftype]
      }
#   insert-debug "help:  looking for file $file"
      if {[file exists $file] && [file isfile $file]} {return $file}

#    finally type manual/task/task.type
      if {$topic == ""} then {
         set file [file join $root $taskname $taskname.$ftype]
#   insert-debug "help:  looking for file $file"
         if {[file exists $file] && [file isfile $file]} {return $file}
      }
   }
   return ""
}


#    call from bf

proc show-document args {
   if {$args == ""} then {docindex_show} \
      else {document_show [first $args]}
}

proc show-manual {} {
   global env
   html_show "" [file join $env(MIDASBASE) manual HOME.html]
}

proc show-manpages {page} {
   global env
   switch [info tclversion] {
      8.0  {html_show "" [file join $env(MIDASBASE) manual tcl8.0tk8.0 $page.htm]}
      8.1  {html_show "" [file join $env(MIDASBASE) manual tcl8.1tk8.1 $page.htm]}
      default {}
   }
}

proc make-frame {} {
   global env started

set rows 30
set cols 120

   :frame [self frame-name] :label "[self frame-title]"  \
       :icon-name [self icon-title]  \
         :show-footer T

   :panel P :layout T
     :colour local text background #d0d0d0
     :colour local text foreground black
     :galley MANUAL :rows $rows :cols $cols
     :colour local text background ""
     :colour local text foreground ""

   :panel B :layout B :padx 20 :pady 10
     :layout H
     :col-gap 20
     :button Back      :command back
     :button Forward   :command forward
     :button Reload    :command reload

   :show T

   :galley_size MANUAL $rows $cols
   :enable [self frame-name] Enter

   set started 1
}


proc :enable-event {W event} {
   if {$event == "Enter"} then {:galley_resize MANUAL}
}

proc load_image_library {} {
   global env

         switch $env(platform)  {
            windows {
               switch -glob $env(os) {
                  "*9*" -
                  "*NT" {load Img1180.dll Img}
                  default {}
               }
            }
            unix  {
               switch $env(os) {
                  SunOS  {load libimg1.1.so Img}
                  Linux  {}
                  OSF1   {}
                  default {}
               }
            }
            default {}
         }
}

proc start args {
   global started env

   if {$started == 0} then {
#       load_image_library
   }

   make-frame
   close-frame
}

proc click-quit args {
   :modify MANUAL :empty T
   close-frame
}

proc finish args {destroy-frame}


#    procedures for showing postscript files
proc ps_show {task file} {
   global psviewer
   :modify MANUAL :strings "Starting $psviewer to view file $file\n\n"
   exec $psviewer $file &
}

proc prn_show {task file} {
   global prnviewer
   :modify MANUAL :strings "Starting $prnviewer to view file $file\n\n"
   exec $prnviewer $file &
}


#    procedures for showing dvi files
proc dvi_show {task file} {
   global dviviewer
   :modify MANUAL :strings "Starting $dviviewer to view file $file\n\n"
   exec $dviviewer $file &
}


#    procedures for showing text files
proc txt_show {task file} {
   global framenumber

#    read the file

   set z [catch {open $file} f]
   if {$z != 0} then {midas-warning $f; return}

   while {[gets $f line] >= 0} {
      if {[string range $line 0 2] == "+++"} then {
         set includefile [findfile $task "" [string range $line 3 end]]
         set z [catch {open $includefile} if]
         if {$z == 0} then {
            append text [read $if]
            close $if
         } else {
            set widget [string range $line 3 end]
            set z [catch {send $task :_help_widget $widget} info]
            if {$z == 0} then {append text $info} \
               else {midas-warning $info}
         }
      } else {
         append text "$line\n"
      }
   }
   close $f
   
     set frame [self frame-name]$framenumber

    :frame $frame :label  "Information on $task from $file" :toplevel T
         wm protocol $frame WM_DELETE_WINDOW "destroy $frame"

    :galley G :deaf T :strings $text :font fixed
    :show T
    
    incr framenumber
}


#    procedure for showing gif files
proc gif_show {task file} {
   global framenumber frame

    image create photo $frame$framenumber -format gif -file $file

    :frame [self frame-name]$framenumber :label  "Image for $task from $file" :transient T :toplevel T

    :canvas C$framenumber :w 256 :h 256
    [:path C$framenumber] create image 128 128 -image $frame$framenumber

    :show T
    incr framenumber
}

#    procedure for showing bmp files
proc bmp_show {task file} {
   global framenumber frame

    image create photo $frame$framenumber -format bmp -file $file

    :frame [self frame-name]$framenumber :label  "Image for $task from $file" :transient T :toplevel T

    :canvas C$framenumber :w 256 :h 256
    [:path C$framenumber] create image 128 128 -image $frame$framenumber

    :show T
    incr framenumber
}



proc docindex_show {} {
   global framenumber documentbase

   set file [file join $documentbase edoc000 index]

#    read the file

   set z [catch {open $file} f]
   if {$z != 0} then {midas-warning $f; return}
   set text [read $f]
   close $f

    :frame [self frame-name]$framenumber :label  "Documentation index from $file" :transient T :toplevel T

    :list DOCINDEX :rows 20 :cols 80 :scroll B :strings [split $text \n]

    :show T
    incr framenumber
}

proc document_show {doc} {
  global documentbase
   foreach format [list html htm dvi ps prn txt gif bmp] {
      set file [file join $documentbase $doc $doc.$format]
      if {[file exists $file]} then {
         if {[midas-confirmation "Show document $file"]} then {
            [set format]_show document $file
            if {$format != "html" && $format != "htm"} then {after 2000 close-frame}
            return
         }
      }
   }
   midas-warning "No suitable document found for $doc"
}

proc click-docindex {v} {document_show  [lowercase [first $v]]}



#    procedures for showing HM files

loadsource html_library-1.11
proc HMtag_table {state win param text} {}
proc HMtag_/table {state win param text} {}


set Running busy                        ;# page status

set NextUrl 0
set CurrentUrl 0
set MaxUrl 0
set Urls(0) ""


# Go render a page.  We have to make sure we don't render one page while
# still rendering the previous one.  If we get here from a recursive
# invocation of the event loop, cancel whatever we were rendering when
# we were called.
# If we have a fragment name, try to go there.

proc html_show {task file} {
   global started
   global myArray htmlcmds Url

        global HM.text
        global Running message

        open-frame
        set widget [:path MANUAL].text

        if {$started == 1} then  {

           HMinit_hyperwin $widget
           set started 2
        }

        set-busy
        set fragment ""
        regexp {([^#]*)#(.+)} $file dummy file fragment
        if {$file == "" && $fragment != ""} {
             HMgoto $widget $fragment
             clear-busy
             clear-footer
             return
       }
        HMreset_win $widget
        set Running busy
        set-footer "Displaying $file"
#        insert-debug "Displaying $file"
        update idletasks
        if {$fragment != ""} {HMgoto $widget $fragment}
        set Url $file


      set lastTag [HMparse_html [HM_get $file] myArray 0 10 incom HMrender start]
      for {set i 1} {$i <= $lastTag} {incr i} {
         regsub -all {\\win\\} $myArray($i) $widget htmlcmds ;
         catch {eval $htmlcmds}
#         insert-debug "$htmlcmds\n\n"
         update
      }

        set Running ready
        HMset_state $widget -stop 1     ;# stop rendering previous page if busy

        clear-busy
        clear-footer
}


# given a file name, return its html, or invent some html if the file can't
# be opened.

proc render {file} {html_show "" $file}

proc back {}  {
    global Urls CurrentUrl NextUrl MaxUrl
    if {$CurrentUrl == 0} then {return}
    incr CurrentUrl -1
    set NextUrl $CurrentUrl
    render $Urls($CurrentUrl)
}

proc forward {}  {
    global Urls CurrentUrl NextUrl MaxUrl
    if {$CurrentUrl == $MaxUrl} then {return}
    incr CurrentUrl 1
    set NextUrl $CurrentUrl
    render $Urls($CurrentUrl)
}

proc reload {}  {
    global Urls CurrentUrl NextUrl MaxUrl
    set NextUrl $CurrentUrl
    render $Urls($CurrentUrl)
}


proc HM_get {file} {
    global Urls CurrentUrl NextUrl MaxUrl
        if {[catch {set fd [open $file]} msg]} {
                return "
                        <title>Bad file $file</title>
                        <h1>Error reading $file</h1><p>
                        $msg<hr>
                "
        }
        set result [read $fd]
        close $fd
        set Urls($NextUrl) $file
        if {$NextUrl > $MaxUrl} then {set MaxUrl $NextUrl}
        set CurrentUrl $NextUrl
        incr NextUrl

        return $result
}

# Supply a link-callback procedure - currently this only handles the simple cases.

proc HMlink_callback {state win href} {
        global Url
        if {[string match #* $href]} {
                render $href
                return
        }
        if {[string match mailto:* $href]} {
            set mailaddr [string range $href 7 end]
                set-footer "Should be sending email to $mailaddr, but isnt yet!"
                return
        }
        if {[string match file:* $href]} {
            set Url [string range $href 5 end]
                render $Url
                return
        }
        if {[string match /* $href]} {
                set Url $href
        } else {
                set Url [file dirname $Url]/$href
        }
        update
        render $Url
}

#  Supply a procedure to implement link connected to a window (button)
set WindowLink(default) ""

proc HMlink_window_setup {b link} {
   global WindowLink
   set WindowLink($b) "$link"
}

# Supply an image callback function
# Read in an image if we don't already have one
# callback to library for display

proc HMset_image {state win handle src} {
        global Url message
        if {[string match /* $src]} {
                set image $src
        } else {
                set image [file dirname $Url]/$src
        }
        set-footer "fetching image $image"
        update
        if {[string first " $image " " [image names] "] >= 0} {
                HMgot_image $handle $image
        } else {
                set type photo
                if {[file extension $image] == ".bmp" || [file extension $image] == ".xbm" } {set type bitmap}
                catch {image create $type $image -file $image} image
                HMgot_image $handle $image
        }
}

# downloading fonts can take a long time.  We'll override the default
# font-setting routine to permit better user feedback on fonts.  We'll
# keep our own list of installed fonts on the side, to guess when delays
# are likely

proc HMset_font {win tag font} {
        global message Fonts
        if {![info exists Fonts($font)]} {
                set Fonts($font) 1
                set-footer "downloading font $font"
                update
        }
        set-footer "Busy..."
        catch {$win tag configure $tag -font $font; $win tag lower $tag mark}
        catch {$win tag configure $tag -kanjifont [HMfont2kfont $font]}
}
